package api

import (
	"bytes"
	"encoding/json"
	"fmt"
	"net/http"
)

type Identifier struct {
	Type string `json:"type"`
	User string `json:"user"`
}

type LoginRequest struct {
	Type                     string     `json:"type"`
	Identifier               Identifier `json:"identifier"`
	Password                 string     `json:"password"`
	RefreshToken             bool       `json:"refresh_token"`
	DeviceId                 string     `json:"device_id"`
	InitialDeviceDisplayName string     `json:"initial_device_display_name"`
}

type LoginResponse struct {
	AccessToken  string `json:"access_token"`
	DeviceId     string `json:"device_id"`
	ExpiresInMs  int    `json:"expires_in_ms"`
	RefreshToken string `json:"refresh_token"`
	UserId       string `json:"user_id"`
}

func Login(username string, password string, deviceId string) (LoginResponse, error) {
	body, err := json.Marshal(LoginRequest{
		Type: "m.login.password",
		Identifier: Identifier{
			Type: "m.id.user",
			User: username,
		},
		Password:                 password,
		RefreshToken:             true,
		DeviceId:                 deviceId,
		InitialDeviceDisplayName: deviceId,
	})
	if err != nil {
		return LoginResponse{}, err
	}
	resp, err := http.Post("https://matrix-client.matrix.org/_matrix/client/v3/login", "application/json", bytes.NewReader(body))
	if err != nil {
		return LoginResponse{}, err
	}
	if resp.StatusCode < 200 || resp.StatusCode >= 300 {
		return LoginResponse{}, fmt.Errorf("request failed %d: %s", resp.StatusCode, resp.Status)
	}
	var loginResponse LoginResponse
	err = json.NewDecoder(resp.Body).Decode(&loginResponse)
	return loginResponse, err
}
